@@ -152,14 +152,21 @@ prune.tree <- function(tree, k=NULL, best=NULL, newdata, nwts,
     k[1] <- -Inf
     deviance <- zp$newdev[ind]
     size <- size[ind]
-    if(!missing(best) && !is.null(best)) index <- ind[sum(best <= size)]
-  } else
-  if(length(k) == 1) index <- sum(k >= alpha)
-  else {
-    k <- pmax(k, -1e+100)
-    ind <- drop(outer(k, alpha, ">=") %*% rep(1, length(alpha)))
-    deviance <- zp$newdev[ind]
-    size <- size[ind]
+    if(!missing(best) && !is.null(best)) {
+      index <- ind[sum(best <= size)]
+      if(length(index) == 0) {
+        warning("best is bigger than tree size")
+        index <- 1
+      }
+    }
+  } else {
+    if(length(k) == 1) index <- sum(k >= alpha)
+    else {
+      k <- pmax(k, -1e+100)
+      ind <- drop(outer(k, alpha, ">=") %*% rep(1, length(alpha)))
+      deviance <- zp$newdev[ind]
+      size <- size[ind]
+    }
   }
   if(index == 1) return(tree)
   if(index > 1) {
@@ -175,7 +182,7 @@ prune.tree <- function(tree, k=NULL, best=NULL, newdata, nwts,
 
 
 predict.tree <- function(object, newdata = list(), 
-	type = c("vector", "tree", "class"), split=F, nwts, eps=1e-3)
+	type = c("vector", "tree", "class"), split=FALSE, nwts, eps=1e-3)
 {
   which.is.max <- function(x)
   {
@@ -200,7 +207,7 @@ predict.tree <- function(object, newdata = list(),
                 as.integer(nf <- dim(frame)[1]),
                 as.integer(dimx[1]),
                 where = double(nf*dimx[1]),
-                NAOK=T)
+                NAOK = TRUE)
     ypred <- matrix(ypred$where, nf)
     dimnames(ypred) <- list(row.names(frame),dimnames(x)[[1]])
     ypred
@@ -242,18 +249,18 @@ predict.tree <- function(object, newdata = list(),
       } else {
 	where <- pred2.tree(object, tree.matrix(newdata))
 	leaf <- frame$var=="<leaf>"
-	frame <- t(where[leaf, , drop=F]) %*% frame$y[leaf]
+	frame <- t(where[leaf, , drop = FALSE]) %*% frame$y[leaf]
 	names(frame) <- names(where)
 	return(frame)
       }
     } else {
       if(!split) {
-	pr <- frame$yprob[where,  , drop = F]
+	pr <- frame$yprob[where,  , drop = FALSE]
 	dimnames(pr)[[1]] <- names(where)
       } else {
 	where <- pred2.tree(object, tree.matrix(newdata))
 	leaf <- frame$var=="<leaf>"
-	pr <- t(where[leaf,,drop=F]) %*% frame$yprob[leaf,,drop=F]
+	pr <- t(where[leaf,,drop = FALSE]) %*% frame$yprob[leaf,,drop=FALSE]
 	dimnames(pr) <- list(names(where), lev)
       }
       if(type=="class") {
@@ -262,7 +269,7 @@ predict.tree <- function(object, newdata = list(),
       } else return(pr)
     }
   # now must be type = "tree"
-  which <- descendants(as.numeric(row.names(frame)))[, where, drop = F]
+  which <- descendants(as.numeric(row.names(frame)))[, where, drop = FALSE]
   if(any(is.na(y))) dev <- rep(NA, nrow(frame))
   else {
     y <- model.extract(newdata, "response")
@@ -288,7 +295,7 @@ predict.tree <- function(object, newdata = list(),
                 )$dev
       dev[which %*% drp > 0] <- NA
     } else {
-#      dev <- -2 * log(1 + which * (frame$yprob[, y, drop = F] - 1)) %*% rep(1, length(y))
+#      dev <- -2 * log(1 + which * (frame$yprob[, y, drop = FALSE] - 1)) %*% rep(1, length(y))
       yp <- frame$yprob
       yp[yp==0] <- max(0,eps)
       drp <- is.na(y); nwts[drp] <- 0; y[drp] <- levels(y)[1]
@@ -331,7 +338,7 @@ pred1.tree <- function(tree, x)
 	      as.integer(dimx[1]),
 	      as.integer(dimx[2]),
 	      where = integer(dimx[1]),
-	      NAOK=T)
+	      NAOK = TRUE)
   ypred <- ypred$where
   names(ypred) <- dimnames(x)[[1]]
   ypred
@@ -344,7 +351,7 @@ na.tree.replace <- function(frame)
     x <- frame[[j]]
     pos <- is.na(x)
     if(any(pos)) {
-      frame <- frame[!pos,  , drop = F]
+      frame <- frame[!pos,  , drop = FALSE]
       warning(paste(sum(pos), 
                     "observations omitted due to missing values in the response"))
     }
@@ -353,7 +360,7 @@ na.tree.replace <- function(frame)
     x <- frame[[j]]
     pos <- is.na(x)
     if(any(pos)) {
-      frame <- frame[!pos,  , drop = F]
+      frame <- frame[!pos,  , drop = FALSE]
       warning(paste(sum(pos), 
                     "observations omitted due to missing values in the supplied weights"))
     }
