@@ -8,21 +8,8 @@ tree.matrix <- function(frame)
   frame$"(weights)" <- NULL
   terms <- attr(frame, "terms")
   if(is.null(terms)) predictors <- names(frame)
-  else {
-    a <- attributes(terms)
-    predictors <- as.character(a$variables)[-1]
-    removals <- NULL
-    if((TT <- a$response) > 0) {
-      removals <- TT
-    }
-    if(!is.null(TT <- a$offset)) {
-      removals <- c(removals, TT)
-    }
-    if(!is.null(removals)) {
-      predictors <- predictors[ -removals]
-      frame <- frame[, -removals]
-    }
-  }
+  else predictors <- as.character(attr(terms, "term.labels"))
+  frame <- frame[predictors]
   term.levels <- lapply(frame, levels)
   factors <- sapply(term.levels, function(x) length(x) > 0)
   if(any(factors)) {
@@ -96,12 +83,12 @@ function(x, pretty = 0, spaces = 2, digits = .Options$digits-3, ...)
 tree.depth <- 
 function(nodes)
 {
-  depth <- floor(log(nodes, base = 2))
+  depth <- floor(log(nodes, base = 2) + 1e-7)
   as.vector(depth - min(depth))
 }
 
 labels.tree <-
-function(object, pretty = T, collapse = T)
+function(object, pretty = TRUE, collapse = TRUE)
 {
   if(!inherits(object, "tree")) stop("Not legitimate tree")
   frame <- object$frame
@@ -181,7 +168,7 @@ function(x, digits = max(.Options$digits - 3, 3), ...)
 }
 
 deviance.tree <-
-function(object, detail = F)
+function(object, detail = FALSE)
 {
   if(!inherits(object, "tree")) stop("Not legitimate tree")
   if(detail) return(object$frame$dev)
@@ -211,7 +198,7 @@ function(object, type = c("usual", "pearson", "deviance"))
 }
 
 misclass.tree <- 
-function(tree, detail = F)
+function(tree, detail = FALSE)
 {
   if(!inherits(tree, "tree"))
     stop("Not legitimate tree")
@@ -238,8 +225,8 @@ function(x, y = NULL, type = NULL, ...)
   if(!inherits(x, "tree")) stop("Not legitimate tree")
   call <- match.call()
   call$type <- NULL
-  if(is.null(type)) uniform <- F
-  else uniform <- if(length(type) == 1 && type == "u") T else F
+  if(is.null(type)) uniform <- FALSE
+  else uniform <- if(length(type) == 1 && type == "u") TRUE else FALSE
   assign(paste(".Tree.unif", dev.cur(), sep = "."), uniform, envir=.GlobalEnv)
   call[[1]] <- as.name("treepl")
   call$x <- treeco(x)
@@ -248,8 +235,8 @@ function(x, y = NULL, type = NULL, ...)
 }
 
 text.tree <- 
-function(x, splits = T, label = "yval", FUN = text, all = F, pretty = NULL, 
-        digits = .Options$digits - 3, ...)
+function(x, splits = TRUE, label = "yval", FUN = text, all = FALSE,
+         pretty = NULL, digits = .Options$digits - 3, ...)
 {
   if(!inherits(x, "tree")) stop("Not legitimate tree")
   frame <- x$frame
@@ -268,7 +255,7 @@ function(x, splits = T, label = "yval", FUN = text, all = F, pretty = NULL,
     ind <- rows != "NA"
     FUN(xy$x[ind], xy$y[ind] + 0.5 * charht, rows[ind], ...)
   }
-  leaves <- if(all) rep(T, nrow(frame)) else frame$var == "<leaf>"
+  leaves <- if(all) rep(TRUE, nrow(frame)) else frame$var == "<leaf>"
   if(label == "yval" & !is.null(ylevels))
     stat <- as.character(frame$yval[leaves])
   else if(!is.null(ylevels) && !is.na(lev <- match(label, ylevels)))
@@ -313,7 +300,7 @@ function(tree, uniform = paste(".Tree.unif", dev.cur(), sep = "."))
 }
 
 treepl <-
-function(x, y, node, erase = F, ...)
+function(x, y, node, erase = FALSE, ...)
 {
   if(!is.null(x$x) && !is.null(x$y)) {
     node <- y
@@ -328,7 +315,7 @@ function(x, y, node, erase = F, ...)
     lines(c(xx[, erase]), c(yy[, erase]), col = par("bg"))
     return(x = x[!erase], y = y[!erase])
   }
-  plot(range(x), range(y), type = "n", axes = F, xlab = "", ylab = "")
+  plot(range(x), range(y), type = "n", axes = FALSE, xlab = "", ylab = "")
   text(x[1], y[1], "|", ...)
   lines(c(xx[, -1]), c(yy[, -1]), ...)
   list(x = x, y = y)
@@ -336,21 +323,21 @@ function(x, y, node, erase = F, ...)
 
 
 snip.tree <-
-function(tree, nodes, xy.save = F, digits = .Options$digits - 3)
+function(tree, nodes, xy.save = FALSE, digits = .Options$digits - 3)
 {
   .snip.tree <- function(tree, nodes)
     {
       where <- tree$where
       frame <- tree$frame
       node <- as.numeric(row.names(frame))
-      if(is.null(frame$which)) frame$which <- descendants(node, F)
+      if(is.null(frame$which)) frame$which <- descendants(node, FALSE)
       i <- match(nodes, node)
       frame$var[i] <- "<leaf>"
       frame$splits[i,  ] <- ""
-      ii <- frame$which[i,  , drop = F]
+      ii <- frame$which[i,  , drop = FALSE]
       keep <- !apply(ii, 2, any)
-      frame <- frame[keep,  , drop = F]
-      frame$which <- frame$which[, keep, drop = F]
+      frame <- frame[keep,  , drop = FALSE]
+      frame$which <- frame$which[, keep, drop = FALSE]
       where <- match(node[where], node[keep])
       nn <- match(nodes, node[keep])
       for(n in seq(length(nodes))) where[ii[n, tree$where]] <- nn[n]
@@ -394,7 +381,7 @@ function(tree, nodes, xy.save = F, digits = .Options$digits - 3)
     i <- node.match(nodes, node, tree$frame$var == "<leaf>")
     if(length(i) == 0) return(tree)
     nodes <- node[i]
-    which <- descendants(node, include = T)
+    which <- descendants(node, include = TRUE)
     nodes <- nodes[apply(which[i,  ] %*% which[, i], 2, "sum") == 1] 
     diag(which) <- F
     tree$frame$which <- which
@@ -431,18 +418,18 @@ identify.tree <- function(x)
 }
 
 descendants <-
-function(nodes, include = T)
+function(nodes, include = TRUE)
 {
   n <- length(nodes)
-  if(n == 1) return(matrix(T, 1, 1))
+  if(n == 1) return(matrix(TRUE, 1, 1))
   ind <- 1:n
-  desc <- matrix(F, n, n)
-  if(include) diag(desc) <- T
+  desc <- matrix(FALSE, n, n)
+  if(include) diag(desc) <- TRUE
   parents <- match((nodes %/% 2), nodes)
   lev <- floor(log(nodes, base = 2))
-  desc[1, 2:n] <- T
+  desc[1, 2:n] <- TRUE
   for(i in max(lev):2) {
-    desc[cbind(ind[parents[lev == i]], ind[lev == i])] <- T
+    desc[cbind(ind[parents[lev == i]], ind[lev == i])] <- TRUE
     parents[lev == i] <- parents[parents[lev == i]]
     lev[lev == i] <- i - 1
   }
@@ -450,7 +437,7 @@ function(nodes, include = T)
 }
 
 node.match <- 
-function(nodes, nodelist, leaves, print.it = T)
+function(nodes, nodelist, leaves, print.it = TRUE)
 {
   node.index <- match(nodes, nodelist, nomatch = 0)
   bad <- nodes[node.index == 0]
@@ -490,14 +477,14 @@ cv.tree <- function(object, rand, FUN = prune.tree, K=10, ...)
 {
   if(!inherits(object, "tree")) stop("Not legitimate tree")
   m <- model.frame(object)
-  extras <- match.call(expand.dots = F)$...
+  extras <- match.call(expand.dots = FALSE)$...
   FUN <- deparse(substitute(FUN))
   init <- do.call(FUN, c(list(object), extras))
-  if(missing(rand)) rand <- sample(K, length(m[[1]]), replace = T)
+  if(missing(rand)) rand <- sample(K, length(m[[1]]), replace = TRUE)
   cvdev <- 0
   for(i in unique(rand)) {
-    tlearn <- tree(model = m[rand != i,  , drop = F])
-    plearn <- do.call(FUN, c(list(tlearn, newdata = m[rand ==i, , drop = F],
+    tlearn <- tree(model = m[rand != i,  , drop = FALSE])
+    plearn <- do.call(FUN, c(list(tlearn, newdata = m[rand ==i, , drop = FALSE],
                                 k = init$k), extras))
     cvdev <- cvdev + plearn$dev
   }
@@ -533,7 +520,7 @@ ptXlines <- function(x, v, xrange, xcoord = NULL, ycoord = NULL, tvar, i = 1)
 }
 
 partition.tree <- 
-function(tree, label = "yval", add = F, ordvars, ...)
+function(tree, label = "yval", add = FALSE, ordvars, ...)
 {
   if(!inherits(tree, "tree")) stop("Not legitimate tree")
   frame <- tree$frame
@@ -591,3 +578,76 @@ function(tree, label = "yval", add = F, ordvars, ...)
     text(yy[1,  ], yy[2,  ], as.character(lab))
   }
 }
+
+tree.screens <-
+function(figs, screen.arg = 0, ...)
+{
+  if(missing(figs))
+    figs <- matrix(c(0,0,1,1,0.25,0.05,1,0.25),2,4)
+  which <- split.screen(figs, screen.arg)
+  par(mar = rep(0, 4), cex = 1, ...)
+  which
+}
+
+
+tile.tree <- 
+function(tree, var, screen.arg = ascr + 1, axes = TRUE)
+{
+  if(!inherits(tree, "tree")) stop("Not legitimate tree")
+  where <- tree$where
+  varname <- substitute(var)
+  v <- match(deparse(varname), as.character(attr(tree$terms, "variables")),
+             nomatch = 0)
+  if(v) var <- model.frame(tree)[, v]
+  else {
+    if(!is.null(data <- data.tree(tree))) {
+      var <- eval(varname, data)
+      names(var) <- row.names(data)
+    } else names(var) <- database.attr("row.names")
+    var <- var[names(where)]
+  }
+  if(length(var) != length(where))
+    stop("Variable length must match that of data used in fit")
+  if(any(is.na(var))) var <- na.tree.replace(list(var = var))$var
+  if(!(levx <- length(levels(var)))) {
+    var <- cut(var, quantile(var) - c(1, 0, 0, 0, 0))
+    levx <- 4
+    names(var) <- names(where)
+  }
+  leaves <- tree$frame$var == "<leaf>"
+  nl <- sum(leaves)
+  x <- seq(nl)
+  indices <- unlist(split(names(where), where))
+  counts <- c(table(var[indices], rep(1:nl, tree$frame$n[leaves])))
+  xm <- max(x)
+  y <- seq(0, 1, length = levx + 1)[-1]
+  dy <- y[2] - y[1]
+  y <- rep(y, nl)[as.logical(counts)]
+  x <- rep(x, rep(levx, nl))[as.logical(counts)]
+  dx <- counts[as.logical(counts)]/(max(counts) * 2)
+  x <- rbind(x, x - dx, x - dx, x, x, NA)
+  y <- rbind(y, y, y + dy, y + dy, y, NA)
+  if(!(ascr <- screen())) stop("Need to invoke split.screen")
+  screen(screen.arg)
+  xpd <- par(xpd = TRUE, mar=rep(0,4))
+  on.exit(par(xpd))
+  plot(x, y, type = "l", axes = FALSE, xlab = "", ylab = "", xlim = c(1, xm))
+  if(axes)
+    abline(v = seq(xm))
+  screen(ascr, new = FALSE)
+  counts <- matrix(counts, levx, nl,
+                   dimnames = list(levels(var), row.names(tree$frame)[leaves]))
+  invisible(counts)
+}
+
+data.tree <-
+function(tree)
+{
+  oc <- tree$call
+  while(oc[[1]] != "tree") oc <- eval(oc[[2]])$call
+  if(is.null(oc$data)) NULL
+  else {
+    warning(paste("Retrieving data from", oc$data))
+    eval(oc$data)
+  }
+}
